package net.pingex.discordbot;

import sx.blah.discord.api.IDiscordClient;
import sx.blah.discord.handle.impl.events.MessageReceivedEvent;
import java.util.Map;

/**
 * Internal commands of the bot, such as `help`, and so on
 * @version 0.1-dev
 * @author Raphael "Pingex" NAAS
 */
@Controllable(name="internal")
class InternalCommandsModule extends AbstractModule
{
	/**
	 * Constructor doing all the basic stuff, like registering as a Listener to Discord, getting a logger, etc.
	 * @param client Discord Client instance used to register self.
	 */
	public InternalCommandsModule(IDiscordClient client)
	{
		super(client);
	}

	/**
	 * Scans and print the usage of a command
	 * @param command Full command, ie `module:command`
	 * @return The man of this command
	 */
	@Command(shorthand = "help", description = "Gives the usage of a command.")
	public String help(MessageReceivedEvent event, String command)
	{
		// Shorthand and lower-case matcher
		if(!command.contains(":") && CommandDispatcher.getInstance().getShortList().containsKey(command.toLowerCase()))
			command = CommandDispatcher.getInstance().getShortList().get(command.toLowerCase());
		else
			command = command.toLowerCase();
		if(!CommandDispatcher.getInstance().getCommandList().containsKey(command)) return "Command not found.";

		InvokableMethod matchingMethod = CommandDispatcher.getInstance().getCommandList().get(command);
		Map<String, String> shorthands = CommandDispatcher.getInstance().getShortList();

		StringBuffer toReturn = new StringBuffer();
		toReturn.append(Configuration.getValue("general", "commandPrefix")).append(command).append(" - ");
		toReturn.append(matchingMethod.getMethod().getAnnotation(Command.class).description());

		// Shorthand
		if(shorthands.containsValue(command))
		{
			toReturn.append(" (shorthand: `");
			for(Map.Entry<String, String> i : shorthands.entrySet())
				if(i.getValue().equals(command))
				{
					toReturn.append(i.getKey());
					break;
				}
			toReturn.append("`)");
		}

		toReturn.append("\nUsage: ").append(Configuration.getValue("general", "commandPrefix")).append(command);

		for(int i=1; i < matchingMethod.getMethod().getParameterCount(); i++)
		{
			// Try to know if the field is required (or not) from the Command annotation
			String brackets = "<>";
			try { if(!matchingMethod.getMethod().getAnnotation(Command.class).required()[i-1]) brackets = "[]"; } catch (ArrayIndexOutOfBoundsException e) {}
			toReturn.append(" ").append(brackets.charAt(0)).append(matchingMethod.getMethod().getParameterTypes()[i].getSimpleName()).append(brackets.charAt(1));
		}

		return toReturn.toString();
	}

	/**
	 * List all commands registered
	 * @param page Page number
	 * @return String representation of this command
	 */
	@Command(shorthand = "list", description = "List all commands available to you.", required = {false})
	public String list(MessageReceivedEvent event, Integer page)
	{
		// TODO: Show only available commands
		Map<String, InvokableMethod> commands = CommandDispatcher.getInstance().getCommandList();
		Map<String, String> shorthands = CommandDispatcher.getInstance().getShortList();
		if(page == null) page = 1;

		int pagesCount = (int) Math.ceil(commands.size()/10.0);
		if(page > pagesCount || page <= 0)
			return "Page #" + page + " doesn't exist. " + pagesCount + " pages are available.";

		StringBuffer toReturn = new StringBuffer("List of commands ");
		toReturn.append("(page ").append(page).append("/").append(pagesCount).append(") ");
		toReturn.append("- Call `/help command` for individual help and usage.\n");

		int pos = 1;
		for(Map.Entry<String, InvokableMethod> i : commands.entrySet())
		{
			if(pos > page*10-10 && pos <= page*10)
			{
				toReturn.append("- ").append(i.getKey());
				toReturn.append("\t").append(i.getValue().getMethod().getAnnotation(Command.class).description());

				// Shorthand
				if(shorthands.containsValue(i.getKey()))
				{
					toReturn.append(" (shorthand: ");
					for(Map.Entry<String, String> j : shorthands.entrySet())
						if(j.getValue().equals(i.getKey()))
						{
							toReturn.append(j.getKey());
							break;
						}
					toReturn.append(")");
				}
				toReturn.append("\n");
			}
			pos++;
		}
		return toReturn.toString();
	}
}
